// BtClientSE.java
// Data exchange with BtServerApp or BtServerSE

import java.io.*;
import ch.aplu.bluetooth.*;
import javax.swing.JOptionPane;
import ch.aplu.util.Console;
import ch.aplu.util.ExitListener;
import javax.bluetooth.*;

public class BtClientSE implements ExitListener
{
  // ------------- Inner class TransceiverThread ---------------
  private class TransceiverThread extends Thread
  {
    public void run()
    {
      Console.delay(2000);  // Wait until server is ready
      try
      {
        for (int n = 1; n <= 100; n++)
        {
          dos.writeInt(n);
          dos.flush();
          c.println("Sent: " + n);

          // Blocking, throws IOException
          // when server closes connection or BluetoothClient.disconnect() is called
          int k = dis.readInt();
          c.println("Got: " + k);
          if (k == disconnectTag)
            throw new IOException("Got disconnect notification");
        }
        dos.writeInt(disconnectTag);  // Send disconnect notification
        dos.flush();
      }
      catch (IOException ex)
      {
        c.println("Transceiver thread: " + ex.getMessage());
      }
      Console.delay(2000);
      bc.disconnect();
      c.println("Transceiver thread terminated");
    }

  }
  // ------------- End of inner class -----------------------

  private final String serviceName = "BtServer";
  private final int disconnectTag = 9999;
  private BluetoothClient bc;
  private DataInputStream dis;
  private DataOutputStream dos;
  private Console c;

  public BtClientSE()
  {
    c = Console.init();
    c.println("Asking for server Bluetooth name");
    c.addExitListener(this);

    String prompt = "Enter Server's Bluetooth Name";
    String serverName;
    do
    {
      serverName = JOptionPane.showInputDialog(null, prompt);
      if (serverName == null)
        System.exit(0);
    }
    while (serverName.trim().length() == 0);
    serverName = serverName.trim();

    c.println("Trying to find paired device '" + serverName + "'");
    RemoteDevice rd = BluetoothFinder.searchPreknownDevice(serverName);
    if (rd == null)
    {
      c.println("Sorry. Device '" + serverName + "' not paired.");
      c.println("Please perform Bluetooth device pairing!");
      return;
    }
    c.println("Paired device found. Searching service now...");
    bc = new BluetoothClient(rd, serviceName);
    bc.setVerbose(true);
    c.println("Trying to connect to '" + serverName
      + "' using service '" + serviceName + "'");
    if (!bc.connect())
    {
      c.println("Connection failed");
      return;
    }
    c.println("Connection established");
    c.println("Retrieving input and output streams");
    dis = new DataInputStream(bc.getInputStream());
    dos = new DataOutputStream(bc.getOutputStream());

    c.println("Starting transceiver thread");
    new TransceiverThread().start();
  }

  // Callback when close button is hit
  public void notifyExit()
  {
    if (bc != null && bc.isConnected())
    {
      try
      {
        dos.writeInt(disconnectTag);
        dos.flush();
      }
      catch (IOException ex)
      {
      }
      bc.disconnect();
    }
    System.exit(0);
  }

  public static void main(String[] args)
  {
    new BtClientSE();
  }

}
